/***********************************************************************************/
/**	\file				ComplexSurface.cpp
**	\brief				Implementation zur Klasse ComplexSurface
*************************************************************************************
**	Autor:				Christian Roesch
*************************************************************************************
**	-tut nichts-
**
*//*********************************************************************************/


// includes
#include "ComplexSurface.h"

#include <sstream>

#include "Config.h"
#include "ScreenModes.h"

// Methoden-Definitionen
namespace FrameWork
{

ComplexSurface::ComplexSurface()
:
m_pSurface( NULL ),
m_bIsInitialized( false )
{
}

ComplexSurface::~ComplexSurface()
{
	free();
}

bool ComplexSurface::load( std::string strFileName )
{
	Config cfgGfxInfo;
	if( !cfgGfxInfo.read( strFileName ) )
	{
		return false;
	}

	int iMode= ScreenModes::inst().getActModeIndex();

	std::ostringstream ossSuffix;
	ossSuffix << "_" << iMode;

	if( cfgGfxInfo.ExistsString( "FILE" + ossSuffix.str() ) )
	{
		m_strFile= cfgGfxInfo.getString( "FILE" + ossSuffix.str() );
	}
	else
	{
		if( cfgGfxInfo.ExistsString( "FILE" ) )
		{
			m_strFile= cfgGfxInfo.getString( "FILE" );
		}
		else
		{
			return false;
		}
	}

	if( cfgGfxInfo.ExistsFloat( "SIZEX" + ossSuffix.str() ) )
	{
		m_fSizeX= cfgGfxInfo.getFloat( "SIZEX" + ossSuffix.str() );
	}
	else
	{
		if( cfgGfxInfo.ExistsFloat( "SIZEX" ) )
		{
			m_fSizeX= cfgGfxInfo.getFloat( "SIZEX" );
		}
		else
		{
			return false;
		}
	}

	if( cfgGfxInfo.ExistsFloat( "SIZEY" + ossSuffix.str() ) )
	{
		m_fSizeY= cfgGfxInfo.getFloat( "SIZEY" + ossSuffix.str() );
	}
	else
	{
		if( cfgGfxInfo.ExistsFloat( "SIZEY" ) )
		{
			m_fSizeY= cfgGfxInfo.getFloat( "SIZEY" );
		}
		else
		{
			return false;
		}
	}

	if( cfgGfxInfo.ExistsFloat( "FRAMES" + ossSuffix.str() ) )
	{
		m_iFrames= (int)cfgGfxInfo.getFloat( "FRAMES" + ossSuffix.str() );
	}
	else
	{
		if( cfgGfxInfo.ExistsFloat( "FRAMES" ) )
		{
			m_iFrames= (int)cfgGfxInfo.getFloat( "FRAMES" );
		}
		else
		{
			return false;
		}
	}

	if( cfgGfxInfo.ExistsFloat( "FRAMESPERLINE" + ossSuffix.str() ) )
	{
		m_iFramesPerLine= (int)cfgGfxInfo.getFloat( "FRAMESPERLINE" + ossSuffix.str() );
	}
	else
	{
		if( cfgGfxInfo.ExistsFloat( "FRAMESPERLINE" ) )
		{
			m_iFramesPerLine= (int)cfgGfxInfo.getFloat( "FRAMESPERLINE" );
		}
		else
		{
			return false;
		}
	}

	if( m_iFrames < 1 )
	{
		m_iFrames= 1;
	}
	if( m_iFramesPerLine < 1 )
	{
		m_iFramesPerLine= 1;
	}

	m_pSurface= new FrameWork::Surface( m_strFile, false );

	m_bIsInitialized= true;
	return true;
}

void ComplexSurface::free()
{
	delete m_pSurface;
	m_pSurface= NULL;
	m_bIsInitialized= false;
}

void ComplexSurface::drawFrame(	FrameWork::Surface* pTarget, 
								const D3DXVECTOR2& vec2Center, 
								const D3DXVECTOR2& vec2Size, 
								int iFrame,
								float fOrientation/* = 0.0f*/, 
								DWORD dwColor/* = D3DCOLOR_XRGB(255, 255, 255)*/ )
{
	if( !m_bIsInitialized )
	{
		return;
	}

	iFrame= iFrame % m_iFrames;

	int Line= iFrame / m_iFramesPerLine;
	int Row= iFrame % m_iFramesPerLine;

	float fPosX= m_fSizeX * (float)Row;
	float fPosY= m_fSizeY * (float)Line;

	pTarget->drawSubSprite( 
		m_pSurface,
		D3DXVECTOR2(fPosX, fPosY),
		D3DXVECTOR2(m_fSizeX, m_fSizeY ),
		vec2Center,
		vec2Size,
		fOrientation,
		dwColor );
}

}
/************************************************************************************
**	Ende der Datei:		ComplexSurface.cpp
************************************************************************************/
