/***********************************************************************************/
/**	\file				Config.cpp
**	\brief				Implementation zur Klasse Config
*************************************************************************************
**	Autor:				Christian Roesch
*************************************************************************************
**	-tut nichts-
**
*//*********************************************************************************/


// includes
#include "Config.h"
#include "StringHelper.h"

#include <fstream>

// Methoden-Definitionen

namespace FrameWork
{

Config::Config()
{
}

bool Config::read( std::string strFileName )

{


	TextFileReader tfrConfig;
	if( !tfrConfig.read( strFileName ) )
	{
		return false;
	}
	
	for( int i= 0; i < tfrConfig.getLineCount(); ++i )
	{
		std::string actLine= tfrConfig.getFileLines()[ i ];
		parseKey( actLine );
	}
	
	return true;
}

bool Config::write(	std::string strFileName )
{
	std::ofstream ofsOut( strFileName.c_str() );

	if( !ofsOut.is_open() )
	{
		return false;
	}

	ofsOut << "#Do not modify" << std::endl;

	std::vector< std::string > vecstrKeys= m_dsConfigData.getKeyList();

	for( int i= 0; i < (int)vecstrKeys.size(); ++i )
	{
		ofsOut << vecstrKeys[ i ] << "=\"";
		ofsOut << m_dsConfigData.getData( vecstrKeys[ i ] )->getString(); 
		ofsOut << "\"" << std::endl;	
	}

	return true;
}

void Config::parseKey( std::string strLine )
{
	StringHelper::trimFront( strLine, StringHelper::getSpaceTab() );
	if( strLine.size() == 0 )
	{
		return;
	}
	
	std::string strKey;
	if( !StringHelper::splitAt( strLine, "=", strKey ) )
	{
		return;
	}
	StringHelper::trimFront( strLine, StringHelper::getSpaceTab() );
	StringHelper::trimBack( strKey, StringHelper::getSpaceTab() );
	
	if( strLine[ 0 ] == '\"' )
	{
		strLine= strLine.substr( 1 );
		
		int iLastChar= (int)strLine.size() - 1 ;
		if( strLine[ iLastChar ] == '\"' )
		{
			strLine= strLine.substr( 0, iLastChar );
		}
	}
	
	StringHelper::toUpper( strKey );

	switch( strKey[ 0 ] )
	{
	case m_cPrefixComment: 
		//Kommentar
		break;
	case m_cPrefixFloat:
		{
			float fData= (float)atof( strLine.c_str() );
			m_dsConfigData.addFloat( strKey, fData );
			break;
		}
	default:
		m_dsConfigData.addString( strKey, strLine );
		break;
	}
}

void Config::addString( std::string strName, std::string strData )
{
	m_dsConfigData.addString( strName, strData );
}

std::string Config::getString( std::string strName )
{
	return m_dsConfigData.getString( strName );
}

void Config::addFloat( std::string strName, float fData )
{
	m_dsConfigData.addFloat( m_cPrefixFloat + strName, fData );
}

float Config::getFloat( std::string strName )
{
	return m_dsConfigData.getFloat( m_cPrefixFloat + strName );
}

bool Config::ExistsString( std::string strName )
{
	return m_dsConfigData.ExistsData( strName );
}

bool Config::ExistsFloat( std::string strName )
{
	return m_dsConfigData.ExistsData( m_cPrefixFloat + strName );
}

}

/************************************************************************************
**	Ende der Datei:		Config.cpp
************************************************************************************/
