/***********************************************************************************/
/**	\file				Ingame.h
**	\brief				Header File zur Klasse Ingame
*************************************************************************************
**	Autor:				Christian Roesch
*************************************************************************************
**	_tut nichts_
**
*//*********************************************************************************/

#ifndef _Ingame_H
#define _Ingame_H

// includes
#include "..\ComplexSurface.h"
#include "..\Font.h"
#include "..\GameState.h"
#include "..\Config.h"
#include "..\Mouse.h"
#include "..\Framework.h"


// Klassen-Deklaration

/***********************************************************************************/
/** \brief	Ingame _tut nichts_
*************************************************************************************
**	Genau genommen _tut dies nichts_
**
*//*********************************************************************************/

class	Ingame : public FrameWork::GameState
{
public:
	Ingame( FrameWork::Config* pSettings );
	~Ingame();

	std::string getName() const;

	void onEnter( void* p_context );
	void onLeave( void* p_context, bool quit );
	void render( float dt );
	void update( float dt );

protected:

private:
	FrameWork::ComplexSurface m_csArrow;
	FrameWork::ComplexSurface m_csBoard;
	FrameWork::ComplexSurface m_csBalls;

	FrameWork::ComplexSurface m_csGameOver;

	FrameWork::Font* m_pFont1;

	float m_fArrowX;
	float m_fArrowY;

	float m_fGameBoardX;
	float m_fGameBoardY;

	float m_fGameBoardStepX;
	float m_fGameBoardStepY;

	float m_fGameLevelX;
	float m_fGameLevelY;

	float m_fGameScoresX;
	float m_fGameScoresY;

	float m_fGameOverX;
	float m_fGameOverY;

	float m_fScreenX;
	float m_fScreenY;
	FrameWork::Mouse m_Mouse;

	static const int m_iBoardLengthX= 10;
	static const int m_iBoardLengthY= 10;
	static const int m_iMaxColors= 8;
	static const int m_iMinChain= 3;
	static const int m_iDeletedPerLevel= 50;
	static const int m_iMoveSpeed= 20;

	struct Point
	{
		int m_iX;
		int m_iY;
		Point( int iX= 0, int iY= 0 )
		{
			m_iX= iX;
			m_iY= iY;
		}
	};

	struct BoardField
	{
		int		m_iColor;
		int		m_iAppeared;

		int		m_iPathStep;

		BoardField( int iColor= -1, int iAppeared= 0 )
		{
			m_iColor= iColor;
			m_iAppeared= iAppeared;
			m_iPathStep= 0;
		}
	};

	struct Board
	{
		Board( int iX, int iY );

		int m_iSizeX;
		int m_iSizeY;
		std::vector< BoardField > m_vecbfData;

		Ingame* m_pIngame;

		void clear();

		BoardField& get( int iX, int iY );
		void set( int iX, int iY, BoardField bfNew );

		bool isFree( int iX, int iY );

		int appear();
		int addBalls( int iCount, int iColors );

		bool findPath( Point ptStart, Point ptEnd );
		void clearPath();

		int move( Point ptStart, Point ptEnd );

		int chain( Point ptStart, int iMinCount, float fTimeOffset= 0 ); 
		int realchain( Point ptStart, int iMinCount, float fTimeOffset ); 
		int tobias( Point ptStart, int iMinCount, float fTimeOffset ); 

		bool checkPlayability();
	};


	Board m_Board;

	bool m_bGameOver;
	int m_iLevel;
	int m_iDeletedBalls;
	int m_iScore;
	int m_iInRow;
	int getBallCount();
	int getColorCount();
	void score( int iDeleted, Point ptPos );

	bool m_bSelected;
	Point m_ptSelection;

	bool getBoardMouseHit( int& iX, int& iY );

	struct TextEffect
	{
		TextEffect()
		{
			m_fTime= 0.0f;
		}

		TextEffect( const TextEffect& te )
		{
			*this= te;
		}

		TextEffect& operator= ( const TextEffect& te )
		{
			m_strText= te.m_strText;
			m_vec2Pos= te.m_vec2Pos;
			m_fTime= te.m_fTime;
			return *this;
		}

		std::string m_strText;
		FrameWork::vec2 m_vec2Pos;
		float m_fTime;
	};
	std::list< TextEffect > m_listTexteffects;

	struct BallEffect
	{
		BallEffect()
		{
			m_fTime= 0.0f;
		}

		BallEffect( const BallEffect& be )
		{
			*this= be;
		}

		BallEffect& operator= ( const BallEffect& be )
		{
			m_ptPos= be.m_ptPos;
			m_iColor= be.m_iColor;
			m_Mode= be.m_Mode;
			m_fTime= be.m_fTime;
			return *this;
		}

		Point m_ptPos;
		int m_iColor;
		enum BE_MODE
		{
			BE_EXPLODE,
			BE_APPEAR
		};
		BE_MODE m_Mode;
		float m_fTime;
	};
	std::list< BallEffect > m_listBalleffects;

	struct BallMove
	{
		BallMove()
		{
			m_fTime= 0.0f;
		}

		BallMove( const BallEffect& be )
		{
			*this= be;
		}

		BallMove& operator= ( const BallMove& bm )
		{
			m_vecptPos= bm.m_vecptPos;
			m_iColor= bm.m_iColor;
			m_fTime= bm.m_fTime;
			return *this;
		}

		std::vector< Point > m_vecptPos;
		int m_iColor;
		float m_fTime;
	};
	std::list< BallMove > m_listBallMoves;

	int m_iRules;

	FrameWork::Config* m_pSettings;
};

#endif//_Ingame_H

class	Ingame;


/************************************************************************************
**	Ende der Datei:		Ingame.h
************************************************************************************/ 
