/***********************************************************************************/
/**	\file				Outro.cpp
**	\brief				Implementation zur Klasse Outro
*************************************************************************************
**	Autor:				Christian Roesch
*************************************************************************************
**	-tut nichts-
**
*//*********************************************************************************/


// includes
#include "Outro.h"
#include "../framework.h"
#include "../ScreenModes.h"

// Methoden-Definitionen

Outro::Outro()
:
m_pFont( NULL ),
m_fTimePassed( 0 )
{
	m_listCredits.push_back(
		Credit( "InChains",
				"a game with no more violence",
				0.0f ) );

	m_listCredits.push_back(
		Credit( "Code",
				"Christian Rsch",
				7.0f ) );
	m_listCredits.push_back(
		Credit( "Framework Code",
				"David Scherfgen",
				12.0f ) );
	m_listCredits.push_back(
		Credit( "additional Framework Code",
				"Christian Rsch",
				17.0f ) );
	m_listCredits.push_back(
		Credit( "useless graphics",
				"Christian Rsch",
				22.0f ) );
	m_listCredits.push_back(
		Credit( "idea",
				"Tobias Hummrich",
				27.0f ) );
	m_listCredits.push_back(
		Credit( "artwork",
				"not (yet) included",
				32.0f ) );
	m_listCredits.push_back(
		Credit( "greetz?",
				"Vom Dinkel der Spelz!",
				37.0f ) );

	m_listCredits.push_back(
		Credit( "thank you for playing\nthis outstanding game",
				"",
				48.0f ) );
}

Outro::~Outro()
{
	delete m_pFont;
	m_pFont= NULL;
}

std::string Outro::getName() const
{
	return "Outro";
}

void Outro::onEnter( void* p_context )
{
	FrameWork::ScreenModes::Mode* pMode= FrameWork::ScreenModes::inst().getActMode();

	m_fScreenX= (float)pMode->m_iWidth;
	m_fScreenY= (float)pMode->m_iHeight;

	m_pFont=	new FrameWork::Font(
						pMode->m_cfgVirtualData.getString( "OUTROFONT" ),
						(int)pMode->getScaledFloatX( "OUTROFONT_X" ), 
						(int)pMode->getScaledFloatY( "OUTROFONT_Y" ),
						FrameWork::FF_BOLD );

	m_fTextY= pMode->getScaledFloatY( "OUTROFONT_Y" );
	m_csBlackGrad.load( "gfx/blackgrad.txt" );

	m_fTimePassed= -2.0f;
}

void Outro::onLeave( void* p_context, bool quit )
{
	delete m_pFont;
	m_pFont= NULL;

	m_csBlackGrad.free();
}

void Outro::render( float dt )
{
	FrameWork::Graphics& graphics = FrameWork::Graphics::inst();
	FrameWork::Surface* pBackBuffer= graphics.getBackBuffer();
	// Back-Buffer leeren (Farbe: R, G, B, A)
	pBackBuffer->clear(FrameWork::color(0.0f, 0.0f, 0.0f, 1.0f));

	std::list< Credit >::iterator it;

	for(	it= m_listCredits.begin();
			it != m_listCredits.end();
			it++ )
	{
		const Credit& crArt= *it;
		float fTime= m_fTimePassed - crArt.m_fTime;

		float fTimeScroll= fTime / 15.0f;
		float fTimeUncover= ( fTime - 4.0f ) / 1.5f;

		if( fTimeUncover > 0 )
		{
			m_pFont->drawText(	pBackBuffer,
								FrameWork::vec2(	m_fScreenX * 0.5f, 
													m_fScreenY * ( 1.0f - fTimeScroll ) ),
								crArt.m_strText2.c_str(),
								FrameWork::DTF_HCENTER|FrameWork::DTF_VCENTER,
								0xffffa0a0 );
			if( fTimeUncover < 2.0f )
			{
				m_csBlackGrad.drawFrame(
					pBackBuffer,
					FrameWork::vec2(	m_fScreenX * ( -0.5f + fTimeUncover ), 
										m_fScreenY * ( 1.0f - fTimeScroll ) ),
					FrameWork::vec2( m_fScreenX, m_fTextY ),
					1 );
				m_csBlackGrad.drawFrame(
					pBackBuffer,
					FrameWork::vec2(	m_fScreenX * ( 0.5f + fTimeUncover ), 
										m_fScreenY * ( 1.0f - fTimeScroll ) ),
					FrameWork::vec2( m_fScreenX, m_fTextY ),
					0 );
					
			}
		}

		m_pFont->drawText(	pBackBuffer,
							FrameWork::vec2(	m_fScreenX * 0.5f, 
												m_fScreenY * ( 1.0f - fTimeScroll ) - m_fTextY ),
							crArt.m_strText1.c_str(),
							FrameWork::DTF_HCENTER|FrameWork::DTF_VCENTER,
							0xff80b0ff );

	}

	graphics.showBackBuffer( 1.0f );
}

void Outro::update( float dt )
{
	FrameWork::Input& input = FrameWork::Input::inst();

	m_fTimePassed+= dt;

	if( input.wasKeyboardKeyPressed( DIK_ESCAPE ) )
	{
		getGame()->quit();
	}

	if( m_fTimePassed > 70.0f )
	{
		getGame()->quit();
	}
}

/************************************************************************************
**	Ende der Datei:		Outro.cpp
************************************************************************************/
