/***********************************************************************************/
/**	\file				ScreenModes.cpp
**	\brief				Implementation zur Klasse ScreenModes
*************************************************************************************
**	Autor:				Christian Roesch
*************************************************************************************
**	-tut nichts-
**
*//*********************************************************************************/


// includes
#include "ScreenModes.h"

#include "Config.h"

#include <sstream>

// Methoden-Definitionen

namespace FrameWork
{

ScreenModes::ScreenModes()
{
	m_iActMode= 0;
}

ScreenModes::~ScreenModes()
{
	for( int i= 0; i < (int)m_vecModes.size(); ++i )
	{
		delete m_vecModes[ i ];
	}
}

bool ScreenModes::read( std::string strFile/*="screen/modes.txt"*/ )
{
	Config cfgModes;
	
	if( !cfgModes.read( strFile ) )
	{
		return false;
	}
	
	int iModeCount= (int)cfgModes.getFloat("MODES");

	m_vecModes.clear();

	for( int i= 0; i < iModeCount; ++i )
	{
		std::ostringstream ossFile;
		ossFile << "MODE" << i << "_FILE";
		std::ostringstream ossName;
		ossName << "MODE" << i << "_NAME";

		Mode* pNewMode= new Mode();
		pNewMode->m_strName= cfgModes.getString( ossName.str() );

		Config cfgActMode;
		cfgActMode.read( cfgModes.getString( ossFile.str() ) );

		pNewMode->m_iWidth= (int)cfgActMode.getFloat( "WIDTH" );
		pNewMode->m_iHeight= (int)cfgActMode.getFloat( "HEIGHT" );
		pNewMode->m_bFullScreen= (int)cfgActMode.getFloat( "FULLSCREEN" ) > 0;

		pNewMode->m_cfgVirtualData.read( cfgActMode.getString( "VIRTUAL" ) );

		pNewMode->m_fVirtualX= pNewMode->m_cfgVirtualData.getFloat( "VIRTUALX" );
		pNewMode->m_fVirtualY= pNewMode->m_cfgVirtualData.getFloat( "VIRTUALY" );

		if( pNewMode->m_fVirtualX == 0 )
		{
			pNewMode->m_fVirtualX= 1.0f;
		}
		
		if( pNewMode->m_fVirtualY == 0 )
		{
			pNewMode->m_fVirtualY= 1.0f;
		}

		m_vecModes.push_back( pNewMode );
	}

	return true;
}

float ScreenModes::Mode::getScaledFloatX( std::string strKey )
{
	float fValue= m_cfgVirtualData.getFloat( strKey );

	fValue*= (float) m_iWidth;
	fValue/= m_fVirtualX;

	return fValue;
}

float ScreenModes::Mode::getScaledFloatY( std::string strKey )
{
	float fValue= m_cfgVirtualData.getFloat( strKey );

	fValue*= (float) m_iHeight;
	fValue/= m_fVirtualY;

	return fValue;
}

ScreenModes::Mode* ScreenModes::getActMode()
{
	if( m_iActMode >= (int)m_vecModes.size() || m_iActMode < 0 )
	{
		return &m_modeDummy;
	}

	return m_vecModes[ m_iActMode ];
}

int ScreenModes::getActModeIndex()
{
	return m_iActMode;
}

void ScreenModes::setActMode( int iIndex )
{
	if( iIndex >= (int)m_vecModes.size() || iIndex < 0 )
	{
		m_iActMode= 0;
	}
	m_iActMode= iIndex;
}
	
int ScreenModes::getModeCount()
{
	return (int)m_vecModes.size();
}

ScreenModes::Mode* ScreenModes::getMode( int iIndex )
{
	if( iIndex >= (int)m_vecModes.size() || iIndex < 0 )
	{
		return &m_modeDummy;
	}

	return m_vecModes[ iIndex ];
}

}

/************************************************************************************
**	Ende der Datei:		ScreenModes.cpp
************************************************************************************/
