/*	                     __      __
	                     \ \    / /
	                      \ \  / /
	         _      _______\ \/ /_______      _
	        / \    / __________________ \    / \
	       (   \  | /        __        \ |  /   )
	     ___\  |  | |       / _\       | |  |  /___
	    |____  |  | |      ( <_        | |  | _____|
	      / \| |  | |       \__/       | |  | |/ \
	     / \ | |  | \__________________/ |  | | / \
	    /_/ \__/   \____________________/   \__/ \_\
	 ___  _   _ ___ __  __   _   _  _ ___   _     ____ __
	|   \| | | / __|  \/  | /_\ | \| |_ _| /_\   |__  /  \
	| |) | |_| \__ \ |\/| |/ _ \| .` || | / _ \    / / () |
	|___/ \___/|___/_|  |_/_/ \_\_|\_|___/_/ \_\  /_(_)__/

	      Framework fr den Spieleprogrammierwettbewerb
	             by David Scherfgen (TomasRiker)
*/

#include <cassert>
#include "safe.h"
#include "animated.h"
#include "animation.h"
#include "animationset.h"
#include "utils.h"
using namespace std;

namespace FrameWork
{
	Animated::Animated(AnimationSet* p_animSet)
	{
		assert(p_animSet);
		this->p_animSet = p_animSet;
		this->p_currentAnim = 0;
		p_timer = new Timer();
	}

	Animated::~Animated()
	{
		safeDelete(p_timer);
	}

	void Animated::setAnimation(const string& name)
	{
		Animation* p_anim = p_animSet->findAnimation(name);
		assert(p_anim);
		if(p_currentAnim == p_anim) return;
		p_currentAnim = p_anim;
		resetAnimation();
	}

	const AnimationFrame& Animated::getCurrentFrame()
	{
		assert(p_currentAnim);
		int numFrames = p_currentAnim->getNumFrames();
		int frame = static_cast<int>(p_timer->getValue());
		if(loop) frame %= numFrames;
		else frame = dus_abs(dus_min(frame, numFrames - 1));
		return p_currentAnim->getFrame(frame);
	}

	void Animated::play(float speed,
						bool loop)
	{
		p_timer->setSpeed(speed * p_currentAnim->getSpeed());
		p_timer->resume();
		this->loop = loop;
	}

	void Animated::stop()
	{
		p_timer->pause();
	}

	void Animated::resume()
	{
		p_timer->resume();
	}

	void Animated::resetAnimation()
	{
		*p_timer = 0.0f;
	}
}
