/*	                     __      __
	                     \ \    / /
	                      \ \  / /
	         _      _______\ \/ /_______      _
	        / \    / __________________ \    / \
	       (   \  | /        __        \ |  /   )
	     ___\  |  | |       / _\       | |  |  /___
	    |____  |  | |      ( <_        | |  | _____|
	      / \| |  | |       \__/       | |  | |/ \
	     / \ | |  | \__________________/ |  | | / \
	    /_/ \__/   \____________________/   \__/ \_\
	 ___  _   _ ___ __  __   _   _  _ ___   _     ____ __
	|   \| | | / __|  \/  | /_\ | \| |_ _| /_\   |__  /  \
	| |) | |_| \__ \ |\/| |/ _ \| .` || | / _ \    / / () |
	|___/ \___/|___/_|  |_/_/ \_\_|\_|___/_/ \_\  /_(_)__/

	      Framework fr den Spieleprogrammierwettbewerb
	             by David Scherfgen (TomasRiker)
*/

#ifndef _INPUT_H
#define _INPUT_H

#define DIRECTINPUT_VERSION 0x0800
#include <dinput.h>
#include <cassert>
#include <string>
#include <vector>
#include "singleton.h"

namespace FrameWork
{
	// Ereignistypen
	enum InputEventType
	{
		IET_KEYBOARD_KEY_DOWN,	// Taste auf Tastatur gedrckt
		IET_KEYBOARD_KEY_UP,	// Taste auf Tastatur losgelassen
		IET_MOUSE_MOVEMENT,		// Maus wurde bewegt
		IET_MOUSE_BUTTON_DOWN,	// Mausknopf gedrckt
		IET_MOUSE_BUTTON_UP		// Mausknopf losgelassen
	};

	// Eingabeereignis
	struct InputEvent
	{
		InputEventType type;

		union
		{
			// fr Taste auf Tastatur
			int key;

			// fr Mausknopf
			int button;

			struct
			{
				// fr Mausbewegung
				int x, y, z;
			};

			// ebenfalls fr Mausbewegung
			int movement[3];
		};
	};

	// Klasse fr Eingabe per Tastatur und Maus
	class Input : public Singleton<Input>
	{
	public:
		Input();
		~Input();

		// aktiviert die Eingabe
		void acquireDevices();

		// deaktiviert die Eingabe
		void unacquireDevices();

		// liefert eine Liste mit Eingabeereignissen
		std::vector<InputEvent> processInput();

		// stellt fest, ob eine Taste gerade gedrckt ist
		bool isKeyboardKeyDown(int key) const {assert(key >= 0 && key < 256); return keyboardKey[key] & 1 ? true : false;}

		// stellt fest, ob eine Taste gedrckt wurde
		bool wasKeyboardKeyPressed(int key) const {assert(key >= 0 && key < 256); return keyboardKey[key] & 2 ? true : false;}

		// stellt fest, ob eine Taste losgelassen wurde
		bool wasKeyboardKeyReleased(int key) const {assert(key >= 0 && key < 256); return keyboardKey[key] & 4 ? true : false;}

		// stellt fest, ob ein Mausknopf gerade gedrckt ist
		bool isMouseButtonDown(int button) const {assert(button >= 0 && button < 8); return mouseButton[button] & 1 ? true : false;}

		// stellt fest, ob ein Mausknopf gedrckt wurde
		bool wasMouseButtonPressed(int button) const {assert(button >= 0 && button < 8); return mouseButton[button] & 2 ? true : false;}

		// stellt fest, ob ein Mausknopf losgelassen wurde
		bool wasMouseButtonReleased(int button) const {assert(button >= 0 && button < 8); return mouseButton[button] & 4 ? true : false;}

		// liefert die Bewegung der Maus auf allen drei Achsen
		void getMouseMovement(int* p_x, int* p_y, int* p_z) const {if(p_x) *p_x = mouseAxis[0]; if(p_y) *p_y = mouseAxis[1]; if(p_z) *p_z = mouseAxis[2];}

	private:
		IDirectInput8* p_dInput;
		IDirectInputDevice8* p_keyboard;
		IDirectInputDevice8* p_mouse;
		int keyboardKey[256];
		int mouseButton[8];
		int mouseAxis[3];
	};
}

#endif
