/*	                     __      __
	                     \ \    / /
	                      \ \  / /
	         _      _______\ \/ /_______      _
	        / \    / __________________ \    / \
	       (   \  | /        __        \ |  /   )
	     ___\  |  | |       / _\       | |  |  /___
	    |____  |  | |      ( <_        | |  | _____|
	      / \| |  | |       \__/       | |  | |/ \
	     / \ | |  | \__________________/ |  | | / \
	    /_/ \__/   \____________________/   \__/ \_\
	 ___  _   _ ___ __  __   _   _  _ ___   _     ____ __
	|   \| | | / __|  \/  | /_\ | \| |_ _| /_\   |__  /  \
	| |) | |_| \__ \ |\/| |/ _ \| .` || | / _ \    / / () |
	|___/ \___/|___/_|  |_/_/ \_\_|\_|___/_/ \_\  /_(_)__/

	      Framework fr den Spieleprogrammierwettbewerb
	             by David Scherfgen (TomasRiker)
*/

#include <cassert>
#include "safe.h"
#include "particleemitter.h"
#include "utils.h"
using namespace std;

namespace FrameWork
{
	ParticleEmitter::ParticleEmitter(const ivec2& spriteStartOnTex,
									 const ivec2& spriteSizeOnTex)
	{
		this->spriteStartOnTex = spriteStartOnTex;
		this->spriteSizeOnTex = spriteSizeOnTex;

		active = true;
		autoDestroy = -1.0f;
		position = D3DXVECTOR2(0.0f, 0.0f);
		velocity = D3DXVECTOR2(0.0f, 0.0f);
		particleRate = 20.0f;
		particlesToEmit = 0.0f;

		// Standardparameter setzen
		base.lifetime = 5.0f;
		base.direction = D3DXVECTOR2(1.0f, 0.0f);
		base.offset = D3DXVECTOR2(0.0f, 0.0f);
		base.speed = 10.0f;
		base.startColor = D3DXCOLOR(1.0f, 1.0f, 1.0f, 0.1f);
		base.endColor = D3DXCOLOR(1.0f, 1.0f, 1.0f, 0.0f);
		base.startOrientation = 0.0f;
		base.endOrientation = 10.0f;
		base.startSize = 16.0f;
		base.endSize = 8.0f;
		base.gravityInfluence = 1.0f;
		base.damping = 1.0f;
		ZeroMemory(&deviance, sizeof(deviance));
	}

	ParticleEmitter::~ParticleEmitter()
	{
	}

	void ParticleEmitter::emit(list<Particle>& output,
							   float dt)
	{
		if(!active) return;

		// Partikelzhler erhhen
		particlesToEmit += dt * particleRate;

		// Automatische Zerstrung?
		if(autoDestroy != -1.0f) autoDestroy -= dt;

		while(particlesToEmit >= 1.0f)
		{
			// Partikel erzeugen
			Particle p;
			p.lifetimeLeft = base.lifetime + floatRand(-deviance.lifetime, deviance.lifetime);
			p.spriteStartOnTex = spriteStartOnTex;
			p.spriteSizeOnTex = spriteSizeOnTex;
			p.position = position - dt * floatRand(0.0f, 1.0f) * velocity;
			p.position += base.offset + D3DXVECTOR2(floatRand(-deviance.offset.x, deviance.offset.x), floatRand(-deviance.offset.y, deviance.offset.y));
			p.velocity = base.direction + D3DXVECTOR2(floatRand(-deviance.direction.x, deviance.direction.x), floatRand(-deviance.direction.y, deviance.direction.y));
			D3DXVec2Normalize(&p.velocity, &p.velocity);
			p.velocity *= base.speed + floatRand(-deviance.speed, deviance.speed);
			p.velocity += velocity;
			p.color = base.startColor + D3DXCOLOR(floatRand(-deviance.startColor.r, deviance.startColor.r),
												  floatRand(-deviance.startColor.g, deviance.startColor.g),
												  floatRand(-deviance.startColor.b, deviance.startColor.b),
												  floatRand(-deviance.startColor.a, deviance.startColor.a));
			D3DXCOLOR endColor = base.endColor + D3DXCOLOR(floatRand(-deviance.endColor.r, deviance.endColor.r),
														   floatRand(-deviance.endColor.g, deviance.endColor.g),
														   floatRand(-deviance.endColor.b, deviance.endColor.b),
														   floatRand(-deviance.endColor.a, deviance.endColor.a));
			p.deltaColor = (endColor - p.color) / p.lifetimeLeft;
			p.orientation = base.startOrientation + floatRand(-deviance.startOrientation, deviance.startOrientation);
			float endOrientation = base.endOrientation + floatRand(-deviance.endOrientation, deviance.endOrientation);
			p.rotation = (endOrientation - p.orientation) / p.lifetimeLeft;
			p.size = base.startSize + floatRand(-deviance.startSize, deviance.startSize);
			float endSize = base.endSize + floatRand(-deviance.endSize, deviance.endSize);
			p.deltaSize = (endSize - p.size) / p.lifetimeLeft;
			p.damping = base.damping + floatRand(-deviance.damping, deviance.damping);
			p.gravityInfluence = base.gravityInfluence + floatRand(-deviance.gravityInfluence, deviance.gravityInfluence);

			output.push_back(p);

			particlesToEmit -= 1.0f;
		}

		// Emitter bewegen
		position += dt * velocity;
	}
}
