/*	                     __      __
	                     \ \    / /
	                      \ \  / /
	         _      _______\ \/ /_______      _
	        / \    / __________________ \    / \
	       (   \  | /        __        \ |  /   )
	     ___\  |  | |       / _\       | |  |  /___
	    |____  |  | |      ( <_        | |  | _____|
	      / \| |  | |       \__/       | |  | |/ \
	     / \ | |  | \__________________/ |  | | / \
	    /_/ \__/   \____________________/   \__/ \_\
	 ___  _   _ ___ __  __   _   _  _ ___   _     ____ __
	|   \| | | / __|  \/  | /_\ | \| |_ _| /_\   |__  /  \
	| |) | |_| \__ \ |\/| |/ _ \| .` || | / _ \    / / () |
	|___/ \___/|___/_|  |_/_/ \_\_|\_|___/_/ \_\  /_(_)__/

	      Framework fr den Spieleprogrammierwettbewerb
	             by David Scherfgen (TomasRiker)
*/

#ifndef _SOUNDINSTANCE_H
#define _SOUNDINSTANCE_H

#include <dsound.h>
#include "dsoundresource.h"

namespace FrameWork
{
	// Flags fr das Setzen von Soundparametern
	enum SoundParameterFlags
	{
		SPF_VOLUME = 1,		// Lautstrke
		SPF_PAN = 2,		// Balance
		SPF_FREQUENCY = 4	// Frequenz
	};

	// Instanz eines Sounds
	class SoundInstance : public DSoundResource
	{
		friend class Sound;

	public:
		void onFreeze();
		void onUnfreeze();

		// spielt den Sound ab
		void play(bool loop = false);

		// stoppt den Sound
		void stop();

		// pausiert den Sound
		void pause();

		// beendet die Pause
		void resume();

		// setzt Lautstrke, Balance und Frequenz des Sounds
		void setParameters(DWORD flags, int volume, int pan, DWORD frequency);

		// liefert Lautstrke, Balance und Frequenz des Sounds
		void getParameters(DWORD flags, int* p_volume, int* p_pan, DWORD* p_frequency) const;

	private:
		SoundInstance(IDirectSoundBuffer* p_sound, bool persist);
		~SoundInstance();

		bool toBeDeleted();

		IDirectSoundBuffer* p_buffer;
		int frozen;
		bool wasPlaying;
		bool persist;
		bool virgin;
		bool paused;
		bool loop;
		int volume;
		int pan;
		DWORD frequency;
	};
}

#endif
