/*	                     __      __
	                     \ \    / /
	                      \ \  / /
	         _      _______\ \/ /_______      _
	        / \    / __________________ \    / \
	       (   \  | /        __        \ |  /   )
	     ___\  |  | |       / _\       | |  |  /___
	    |____  |  | |      ( <_        | |  | _____|
	      / \| |  | |       \__/       | |  | |/ \
	     / \ | |  | \__________________/ |  | | / \
	    /_/ \__/   \____________________/   \__/ \_\
	 ___  _   _ ___ __  __   _   _  _ ___   _     ____ __
	|   \| | | / __|  \/  | /_\ | \| |_ _| /_\   |__  /  \
	| |) | |_| \__ \ |\/| |/ _ \| .` || | / _ \    / / () |
	|___/ \___/|___/_|  |_/_/ \_\_|\_|___/_/ \_\  /_(_)__/

	      Framework fr den Spieleprogrammierwettbewerb
	             by David Scherfgen (TomasRiker)
*/
// modified 12.11.05 (TGGC)

#include <cstdio>
#include <cstdlib>
#include <sstream>
#include "utils.h"
#include "graphics.h"
using namespace std;

namespace FrameWork
{
	int intRand(int min,
				int max)
	{
		if(min > max) return intRand(max, min);
		return min + (rand() % (max + 1));
	}

	float floatRand(float min,
					float max)
	{
		if(min > max) return floatRand(max, min);
		return static_cast<float>(min) + (static_cast<float>((rand() % 10001)) / 10000.0f) * (static_cast<float>(max) - static_cast<float>(min));
	}

	bool lineHitsLine(const D3DXVECTOR2& p1,
					  const D3DXVECTOR2& p2,
					  const D3DXVECTOR2& q1,
					  const D3DXVECTOR2& q2,
					  D3DXVECTOR2* p_outIntersection,
					  float* p_outS,
					  float* p_outT)
	{
		const D3DXVECTOR2 p(p1);
		const D3DXVECTOR2 q(q1);
		const D3DXVECTOR2 u(p2 - p1);
		const D3DXVECTOR2 v(q2 - q1);

		const float denominator = u.y * v.x - u.x * v.y;
		if(fabsf(denominator) > 1.0e-8f)
		{
			// Die Linien sind nicht parallel!
			const float s = (p.x * v.y - p.y * v.x - q.x * v.y + q.y * v.x) / denominator;
			if(s >= 0.0f && s <= 1.0f)
			{
				const float t = (p.x * u.y - p.y * u.x - q.x * u.y + q.y * u.x) / denominator;
				if(t >= 0.0f && t <= 1.0f)
				{
					if(p_outIntersection) *p_outIntersection = p + s * u;
					if(p_outS) *p_outS = s;
					if(p_outT) *p_outT = t;
					return true;
				}
			}
		}
		else
		{
			// Die Linien sind parallel! Liegen sie auf derselben Geraden?
			const D3DXVECTOR2 x(q1 - p1);
			if(fabsf(u.y * x.x - u.x * x.y) < 1.0e-8f)
			{
				// Wir projizieren beide auf die erste Linie.
				float s_q1, s_q2, s_v;
				if(fabsf(u.x) > fabsf(u.y))
				{
					s_q1 = (q1.x - p1.x) / u.x;
					s_q2 = (q2.x - p1.x) / u.x;
					s_v = v.x / u.x;
				}
				else
				{
					s_q1 = (q1.y - p1.y) / u.y;
					s_q2 = (q2.y - p1.y) / u.y;
					s_v = v.y / u.y;
				}

				float s;
				if(s_q1 < s_q2)
				{
					if(s_q1 > 1.0f || s_q2 < 0.0f) return false;
					s = s_q1 < 0.0f ? 0.0f : s_q1;
				}
				else
				{
					if(s_q2 > 1.0f || s_q1 < 0.0f) return false;
					s = s_q2 < 0.0f ? 0.0f : s_q2;
				}

				if(s >= 0.0f && s <= 1.0f)
				{
					const float t = (s - s_q1) / s_v;
					if(t >= 0.0f && t <= 1.0f)
					{
						if(p_outIntersection) *p_outIntersection = p + s * u;
						if(p_outS) *p_outS = s;
						if(p_outT) *p_outT = t;
						return true;
					}
				}
			}
		}

		return false;
	}

	bool lineHitsCircle(const D3DXVECTOR2& p1,
						const D3DXVECTOR2& p2,
						const D3DXVECTOR2& center,
						float radius,
						D3DXVECTOR2* p_outIntersection,
						float* p_outS)
	{
		const D3DXVECTOR2 p(p1);
		const D3DXVECTOR2 u(p2 - p1);
		const D3DXVECTOR2 m(center);
		const D3DXVECTOR2 pMinusM(p - m);
		const float r = radius;
		const float rSq = r * r;
		const float uSq = D3DXVec2LengthSq(&u);
		const float uDotPMinusM = D3DXVec2Dot(&u, &pMinusM);

		// Die Diskriminante berechnen:
		//   (u * (p - m))
		// - (u * ((p - m) - r)) 
		const float d = uDotPMinusM * uDotPMinusM
					  - uSq * (D3DXVec2LengthSq(&pMinusM) - rSq);

		// Ist die Diskriminante negativ, null oder positiv?
		if(d < 0.0f) return false;
		else if(d < 1.0e-8f)
		{
			// Die Diskriminante ist (ungefhr) null.
			// Die gesamte Wurzel entfllt und die Lsung ist (-u * (p - m)) / u.
			// Wir mssen nur noch prfen, ob der Wert im korrekten Bereich [0; 1] liegt.
			const float s = -uDotPMinusM / uSq;
			if(s >= 0.0f && s <= 1.0f)
			{
				// Berhrpunkt!
				if(p_outIntersection) *p_outIntersection = p + s * u;
				if(p_outS) *p_outS = s;
				return true;
			}
		}
		else
		{
			// Die Gerade schneidet den Kreis zweimal.
			// Wir suchen die kleinste fr die Linie gltige Lsung.
			const float sqrtD = sqrtf(d);
			const float s1 = (-uDotPMinusM - sqrtD) / uSq;
			if(s1 >= 0.0f && s1 <= 1.0f)
			{
				if(p_outIntersection) *p_outIntersection = p + s1 * u;
				if(p_outS) *p_outS = s1;
				return true;
			}
			else
			{
				const float s2 = (-uDotPMinusM + sqrtD) / uSq;
				if(s2 >= 0.0f && s2 <= 1.0f)
				{
					if(p_outIntersection) *p_outIntersection = p + s2 * u;
					if(p_outS) *p_outS = s2;
					return true;
				}
			}
		}

		return false;
	}

	bool circleHitsCircle(const D3DXVECTOR2& center1,
						  float radius1,
						  const D3DXVECTOR2& center2,
						  float radius2)
	{
		const D3DXVECTOR2 temp(center2 - center1);
		const float d = D3DXVec2LengthSq(&temp);
		return d <= (radius1 + radius2) * (radius1 + radius2);
	}

	bool pointInCircle(const D3DXVECTOR2& point,
					   const D3DXVECTOR2& center,
					   float radius)
	{
		const D3DXVECTOR2 temp(center - point);
		const float d = D3DXVec2LengthSq(&temp);
		return d <= radius * radius;
	}

	void resolveCollision(const D3DXVECTOR2& centerA,
						  const D3DXVECTOR2& velocityA,
						  float massA,
						  const D3DXVECTOR2& centerB,
						  const D3DXVECTOR2& velocityB,
						  float massB,
						  D3DXVECTOR2* p_outVelocityA,
						  D3DXVECTOR2* p_outVelocityB)
	{
		D3DXVECTOR2 v(centerB - centerA);
		D3DXVec2Normalize(&v, &v);

		const D3DXVECTOR2 velocityA1(v * D3DXVec2Dot(&v, &velocityA));
		const D3DXVECTOR2 velocityA2(velocityA - velocityA1);
		const D3DXVECTOR2 velocityB1(v * D3DXVec2Dot(&v, &velocityB));
		const D3DXVECTOR2 velocityB2(velocityB - velocityB1);

		if(p_outVelocityA) *p_outVelocityA = (((2.0f * massB * velocityB1) + (massA - massB) * velocityA1) / (massA + massB)) + velocityA2;
		if(p_outVelocityB) *p_outVelocityB = (((2.0f * massA * velocityA1) + (massB - massA) * velocityB1) / (massA + massB)) + velocityB2;
	}

	void errorMsg(const std::string& text,
				  const std::string& file,
				  int line)
	{
		HWND window = HWND_DESKTOP;
		if(Graphics::isInitialized()) window = Graphics::inst().getWindow();
		/*char tmp[32]; sprintf(tmp, "%d", line);
		string msg = text + "\n\nDatei: " + file + "\nZeile: " + tmp;*/

		std::ostringstream ossMsg;
		ossMsg << text << "\n\nDatei: " << file << "\nZeile: " << line;
		MessageBox(window, ossMsg.str().c_str(), "Fehler", MB_OK | MB_ICONEXCLAMATION);
	}
}
