/*	                     __      __
	                     \ \    / /
	                      \ \  / /
	         _      _______\ \/ /_______      _
	        / \    / __________________ \    / \
	       (   \  | /        __        \ |  /   )
	     ___\  |  | |       / _\       | |  |  /___
	    |____  |  | |      ( <_        | |  | _____|
	      / \| |  | |       \__/       | |  | |/ \
	     / \ | |  | \__________________/ |  | | / \
	    /_/ \__/   \____________________/   \__/ \_\
	 ___  _   _ ___ __  __   _   _  _ ___   _     ____ __
	|   \| | | / __|  \/  | /_\ | \| |_ _| /_\   |__  /  \
	| |) | |_| \__ \ |\/| |/ _ \| .` || | / _ \    / / () |
	|___/ \___/|___/_|  |_/_/ \_\_|\_|___/_/ \_\  /_(_)__/

	      Framework fr den Spieleprogrammierwettbewerb
	             by David Scherfgen (TomasRiker)
*/

#ifndef _UTILS_H
#define _UTILS_H

#include <string>
#include <d3dx9.h>

#define ERROR_MSG(x) {errorMsg((x), __FILE__, __LINE__);}

namespace FrameWork
{
	// Minimum, Maximum und Betrag
	// ***************************

	template<typename T> const T& dus_min(const T& a, const T& b) {return a < b ? a : b;}
	template<typename T> const T& dus_max(const T& a, const T& b) {return a > b ? a : b;}
	template<typename T> T dus_abs(const T& x) {return x < 0 ? x : -x;}


	// Zufallsfunktionen
	// *****************

	// liefert eine zufllige Ganzzahl
	int intRand(int min, int max);

	// liefert eine zufllige Fliekommazahl
	float floatRand(float min, float max);


	// Kollisionserkennung
	// *******************

	// Testet, ob sich zwei Linien schneiden.
	// *p_outIntersection sind die Koordinaten des Schnittpunkts,
	// *p_outS gibt die Position des Schnittpunkts relativ zur ersten Linie,
	// *p_outT relativ zur zweiten Linie an.
	bool lineHitsLine(const D3DXVECTOR2& p1, const D3DXVECTOR2& p2, const D3DXVECTOR2& q1, const D3DXVECTOR2& q2, D3DXVECTOR2* p_outIntersection = 0, float* p_outS = 0, float* p_outT = 0);

	// Testet, ob eine Linie einen Kreis schneidet.
	// *p_outIntersection sind die Koordinaten des ersten Schnittpunkts (falls es zwei gibt),
	// *p_outS gibt die Position des Schnittpunkts relativ zur Linie an.
	bool lineHitsCircle(const D3DXVECTOR2& p1, const D3DXVECTOR2& p2, const D3DXVECTOR2& center, float radius, D3DXVECTOR2* p_outIntersection = 0, float* p_outS = 0);

	// testet, ob sich zwei Kreise schneiden
	bool circleHitsCircle(const D3DXVECTOR2& center1, float radius1, const D3DXVECTOR2& center2, float radius2);

	// testet, ob ein Punkt in einem Kreis liegt
	bool pointInCircle(const D3DXVECTOR2& point, const D3DXVECTOR2& center, float radius);

	// berechnet die Auswirkung einer Kollision zwischen zwei kreisfrmigen Objekten
	void resolveCollision(const D3DXVECTOR2& centerA, const D3DXVECTOR2& velocityA, float massA, const D3DXVECTOR2& velocityB, const D3DXVECTOR2& centerB, float massB, D3DXVECTOR2* p_outVelocityA, D3DXVECTOR2* p_outVelocityB);


	// Fehlermeldungen
	// ***************

	// zeigt eine Fehlermeldung an
	void errorMsg(const std::string& text, const std::string& file, int line);
}

#endif
